/* l7_port.h */

#ifndef __L7_PORT_H
#define __L7_PORT_H

#define        _SIO            _IO
#define        _SIOR           _IOR
#define        _SIOW           _IOW
#define        _SIOWR          _IOWR


#define SNDCTL_DSP_GETTRIGGER          _SIOR ('P',16, int)
#define SNDCTL_DSP_SETTRIGGER          _SIOW ('P',16, int)

#      define PCM_ENABLE_INPUT         0x00000001
#      define PCM_ENABLE_OUTPUT        0x00000002

typedef struct count_info {
               int bytes;      /* Total # of bytes processed */
               int blocks;     /* # of fragment transitions since last time */
               int ptr;        /* Current DMA pointer value */
       } count_info;

#define SNDCTL_DSP_GETIPTR             _SIOR ('P',17, count_info)
#define SNDCTL_DSP_GETOPTR             _SIOR ('P',18, count_info)


/********************************************
 * IOCTL commands for /dev/dsp and /dev/audio
 */

#define SNDCTL_DSP_RESET               _SIO  ('P', 0)
#define SNDCTL_DSP_SYNC                        _SIO  ('P', 1)
#define SNDCTL_DSP_SPEED               _SIOWR('P', 2, int)
#define SNDCTL_DSP_STEREO              _SIOWR('P', 3, int)
#define SNDCTL_DSP_GETBLKSIZE          _SIOWR('P', 4, int)
#define SNDCTL_DSP_SAMPLESIZE          SNDCTL_DSP_SETFMT
#define SNDCTL_DSP_CHANNELS            _SIOWR('P', 6, int)
#define SOUND_PCM_WRITE_CHANNELS       SNDCTL_DSP_CHANNELS
#define SOUND_PCM_WRITE_FILTER         _SIOWR('P', 7, int)
#define SNDCTL_DSP_POST                        _SIO  ('P', 8)
#define SNDCTL_DSP_SUBDIVIDE           _SIOWR('P', 9, int)
#define SNDCTL_DSP_SETFRAGMENT         _SIOWR('P',10, int)


/*
 * ioctl calls to be used in communication with coprocessors and
 * DSP chips.
 */

typedef struct copr_buffer {
               int command;    /* Set to 0 if not used */
               int flags;
#define CPF_NONE               0x0000
#define CPF_FIRST              0x0001  /* First block */
#define CPF_LAST               0x0002  /* Last block */
               int len;
               int offs;       /* If required by the device (0 if not used) */

               unsigned char data[4000]; /* NOTE! 4000 is not 4k */
       } copr_buffer;

typedef struct copr_debug_buf {
               int command;    /* Used internally. Set to 0 */
               int parm1;
               int parm2;
               int flags;
               int len;        /* Length of data in bytes */
       } copr_debug_buf;

typedef struct copr_msg {
               int len;
               unsigned char data[4000];
       } copr_msg;

#define SNDCTL_COPR_RESET             _SIO  ('C',  0)
#define SNDCTL_COPR_LOAD             _SIOWR('C',  1, copr_buffer)
#define SNDCTL_COPR_RDATA            _SIOWR('C',  2, copr_debug_buf)
#define SNDCTL_COPR_RCODE            _SIOWR('C',  3, copr_debug_buf)
#define SNDCTL_COPR_WDATA            _SIOW ('C',  4, copr_debug_buf)
#define SNDCTL_COPR_WCODE            _SIOW ('C',  5, copr_debug_buf)
#define SNDCTL_COPR_RUN                      _SIOWR('C',  6, copr_debug_buf)
#define SNDCTL_COPR_HALT             _SIOWR('C',  7, copr_debug_buf)
#define SNDCTL_COPR_SENDMSG          _SIOWR('C',  8, copr_msg)
#define SNDCTL_COPR_RCVMSG           _SIOR ('C',  9, copr_msg)


/****************************************************************/
#define PROT_READ      0x1             /* page can be read */
#define PROT_WRITE     0x2             /* page can be written */
#define PROT_EXEC      0x4             /* page can be executed */
#define PROT_NONE      0x0             /* page can not be accessed */

#define MAP_SHARED     0x01            /* Share changes */
#define MAP_PRIVATE    0x02            /* Changes are private */
#define MAP_TYPE       0x0f            /* Mask for type of mapping */
#define MAP_FIXED      0x10            /* Interpret addr exactly */
#define MAP_ANONYMOUS  0x20            /* don't use a file */


/***********************************/


typedef signed char s8;
typedef unsigned char u8;

typedef signed short s16;
typedef unsigned short u16;

typedef signed int s32;
typedef unsigned int u32;
/*********************************/
#define	EFAULT		14	/* Bad address */


#endif  // __L7_PORT_H
