/* ldbios.cpp */

/* L-780 bios loader */
/* (C) L-Card, Inc. 2000 */
/* OS/2 */


#define INCL_DOS
#define INCL_DOSERRORS

#include <os2.h>

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <conio.h>
#include <io.h>
#include <string.h>
#include "getopt.h"

#include "l780os2.h"
#include "xdsplib.h"
#include "lbios.h"

char *dname[32] = {"DSP",NULL};
char *bio_path = "/usr/share/lcard";  /* Default BIOS path */
char *bio_name = "l780.bio";
int devices = 0;
BOARD_DESCR descr;
char namebuf[128];
int probe = 0;
int size_pm = 0;
int size_dm = 0;
PM_WORD pm[0x1000];
DM_WORD dm[0x10000];

int TestTTL(int fdev);

char *usage =
 "Usage: ldbios [options] device_name\n"
 "Options:\n"
 " -f NAME  Use bios file NAME\n"
 " -d DIR   Default BIOS path\n";

int ReadBios(FILE * fp)
{
   unsigned short cp, cd;
   int l, rc;

   cp = cd = 0x7FFF;
   rc = fread(&cp, sizeof(short),1, fp);
   if(rc != 1 || cp>0x4000)
          return -1;
   l = cp * sizeof(short);
//   size_pm = 0x1000;
   size_pm = cp;
   rc = fread(pm, l,1,fp);
   if(rc != 1)
          return -1;
   rc = fread(&cd, sizeof(short),1,fp);
   if(rc != 1 || cd>0x4000)
          return -1;
   l = cd * sizeof(short);
//   size_dm = 0x2000;
   size_dm = cd;
   rc = fread(dm, l,1,fp);
   if(rc != 1) return -1;
   return 0;
}

/* Load BIOS from file */

int LoadBiosFile(int dev, char *fname)
{
   unsigned short cp, cd;
   int l, c;

   FILE *fp;
   fp = fopen(fname, "rb");
   if(fp  == NULL) return  1;

   c = ReadBios(fp);
   fclose(fp);
 if(c==0)
 {
  c = xdspReset(dev);
   xdspLoad(dev, 0x03, dm, size_dm, sizeof(DM_WORD), 0x2000 /* 0 */);
   xdspLoad(dev, 0x01, pm + 1, size_pm - 1, sizeof(PM_WORD), 1);
   xdspLoad(dev, 0x01, pm, 1, sizeof(PM_WORD), 0);

 }
   return c;
}

//extern int getopt (int argc, char *const *argv, const char *shortopts);
int main_1(int n, char *par[]);

//static BOARD_STRUCT *FirstBoard = NULL;

int main(int argc, char **argv)
{
 int fdev;
 int x, i, c,rc;
 int id;
 BOARD_DESCR board_descr;

 if(argc > 1)
 {
    while(1)
    {
     c = getopt(argc, argv, "pf:d:");
     if(c == -1) break;
     switch(c)
     {
      case 'p':
      probe = 1;
      break;

      case 'f':
      bio_name = optarg;
      break;

      case 'd':
      bio_path = optarg;
      break;

      default:
      printf("Unknown option\n");
      puts(usage);
      exit(1);

     }
    }
 }

// main_1(argc, argv);

 for(i=optind; i<argc; i++) dname[devices++] = argv[i];

 devices=1;

 for(i=0; i<devices; i++)
 {

  fdev = xdspOpen(dname[i], 0);
  if(fdev<0)
  {
   if(!probe) printf("Unable to open device %s\n", dname[i]);
   continue;
  }


//  if(probe)  /* Test if BIOS is already loaded */
  {
   x = xdspTest(fdev, 0x0);
   if(x == 0)
   {  printf(" L780 襫\n");
   } else {
      printf(" L780  襫\n");
   }
   x = xdspTest(fdev, 0x80);
   if(x)
   {
      printf("L780 த  㦥 ࠡ⠥\n");
//        xdspClose(fdev);
//        continue;
   }
  }

/* Get board ID */

  id = xdspGetId(fdev);
  if(id<0)
  {
   puts("Unable to get board ID!");
   xdspClose(fdev);
   continue;
  }
  else
  {
   printf("\n%s: ID %04X\n", dname[i], id);
  }
/*
// , ᮤঠ ଠ  ,  ࠭  . 
typedef struct _BOARD_DESCR
{
 char serial[9];       // ਩ 
 char name[5];         //  
 char revision;            // ॢ 
 char dsp_type[5];     // ⨯ ⠭ DSP
 long freq;            //    
 unsigned short is_dac;        // 䫠  
 unsigned short _res[7];   // १ࢨ஢
} BOARD_DESCR;

*/

  rc = xdspGetInfo(fdev, (void *)&board_descr);
  if(rc == 0)
  {  printf("਩  %s,  %s, ॢ %c\n",
               board_descr.serial,board_descr.name,board_descr.revision);
     printf("⨯ ⠭ DSP %s,     %i, 䫠   %i\n",
               board_descr.dsp_type,board_descr.freq,board_descr.is_dac);
  }


/* Default BIOS file */

  if(bio_name==NULL)
  {
   sprintf(namebuf, "%s/L%X.bio", bio_path, id);
  }
  else strcpy(namebuf, bio_name);


/* Load BIOS */

  printf("BIOS file: %s\n", namebuf);
  if(LoadBiosFile(fdev, namebuf))
  {
   puts("Unable to load BIOS\n");
   continue;
  }
  printf("%s Load Ok", namebuf);
/* Test it */

  if(xdspTest(fdev, 0) < 0)
  {
   puts("Test failed!\n");
  } else {
     printf("Test Ok");
  }

 if(argc <= 1)
        TestTTL(fdev);


  xdspClose(fdev);
 }
 return 0;
}

int LS_PowerMetrTest(int fdev);

int TestTTL(int fdev)
{  int val_out=0, val_in = 0,rc;
   int pf_csc = 0,pf_d=0,pf_csc_in,pfd_in;
   int v,Channel=0;
   char str[80];
   int i,j;

M: val_in =  ttl_in(fdev);
   pf_csc_in = pflagCSC_in(fdev);
   pfd_in     = pflag_in(fdev);

   v = getADC(fdev, Channel);

   printf("Input TTL=%x, pf_csc=%x, pfd=%x, ADC=%x(%i) \n",val_in,pf_csc_in,pfd_in,v,v);
   printf("1   - TTL out=%x\n",val_out);
   printf("2   - LogChanADC =%x\n",Channel);
   printf("3   -   \n");

   printf("0,Q - 室\n"); fflush(stdout);
   rc = _getch();
   switch(rc)
   {
      case '0':
      case 'q':
      case 'Q':
         return 0;
      case '1':
       printf(" (hex) = "); fflush(stdout);
       gets(str);
       sscanf(str,"%x",&val_out);
       rc = ttl_out(fdev,val_out);
      printf("rc=%x ",rc);
       break;

      case '2':
       printf(" (hex) = "); fflush(stdout);
       gets(str);
       sscanf(str,"%x",&Channel);
       break;


      case '3':
      {   int nchan=0;
          int mode=0;
          int modedif=0, modezero=0;
          int NN = 100, rc00;
          short int curv;
          float mv,mvs;

          printf("  ? "); fflush(stdout);
          do
          {      gets(str);
          } while (str[0] == 0);

          sscanf(str,"%i",&nchan);
          printf(" 16/32. (0/1) ? "); fflush(stdout);
          do
          {      gets(str);
          } while (str[0] == 0);
          sscanf(str,"%i",&modedif);
          if(modedif) modedif = 1;
          printf("஢  (/ = 0/1) ? "); fflush(stdout);
          gets(str);
          sscanf(str,"%i",&modezero);
          if(modezero) modezero=1;
          printf("ᨫ 0/1/2/3 = 5/1.25/0.3125/0.0078) ? "); fflush(stdout);
          gets(str);
          sscanf(str,"%i",&mode);
          mode = mode &0x3;
          if(modezero) nchan &= 0xf;
          Channel  = nchan | (modezero<<4) | (modedif<<5) |(mode <<6);
          printf("᪨   %x\n",Channel);
          printf("\np, p ?\n"); fflush(stdout);
          scanf("%i", &NN);

          printf("Press Q for exit, M for mode change, any key for stop...\n");

          for(i=0; ;i++)
          {   DosSleep(100);
              mvs = 0.;
             for(j=0;j<NN;j++)
             {
                curv = getADC(fdev, Channel);
                mv = float(curv) / 8000.;
                if(mode == 0)      mv *= 5.;
                else if(mode == 1) mv *= 1.25;
                else if(mode == 2) mv *= 0.3125;
                else if(mode == 3) mv *= 0.0078;
                mvs += mv;
             }
            mvs = mvs / double(NN);
            printf("%8.4f",  mvs);
       if(mode == 1)
           printf("\r");
       else
           printf("\n");
       fflush(stdout);
           if(_kbhit())
           {   rc = _getch();
               if(rc == 'q' || rc == 'Q')
                            break;
             if(rc == 'm' || rc == 'M')
             {    mode = (mode +1) & 0x01;
                 continue;
             }

             if(rc == 'q' || rc == 'Q') rc = 2;
             else rc = 1;
            fflush(stdout);

             rc = 0;
              do
              {  if(_kbhit())
                 {  rc = _getch();
                 }
              } while (!rc);
              if(rc == 2) break;
            } /* endof if(_kbhit()) */

          }
      }
       break;

      default:
       rc = ttl_out(fdev,val_out);
   }
   goto M;


}
