/* xdsplib.cpp */
/* ADC/DAC i/o library example */
#define POKA 0

#define INCL_DOS
#define INCL_DOSERRORS
  #include <os2.h>

#include <stdlib.h>
#include <string.h>
//#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "l780_user.h"
#include "xdsplib.h"

extern HFILE hfPCI;

/* Open device */

int xdspOpen(char *Name, int Mode)
{  int rc;
   rc = InitAccess_to_PCI();
   if(rc == NULL)
               return 0;
   rc = init_module();
   return rc;
}

/* Close device */

int xdspClose(int dev)
{
      CloseAccess_to_PCI();
}

/* Input */

int xdspRead(int dev, void *buf, int c)
{
#if POKA
 return read(dev, buf, c);
#endif
}

/* Output */

int xdspWrite(int dev, void *buf, int c)
{
#if POKA
 return write(dev, buf, c);
#endif //POKA
}

/* Start/stop ADC */

int xdspEnableInput(int dev, int Enable)
{
 int x, c;
#if POKA

 c = ioctl(dev, SNDCTL_DSP_GETTRIGGER, &x);
 if(c<0) return c;
 if(Enable) x |= PCM_ENABLE_INPUT;
 else x &= ~PCM_ENABLE_INPUT;
 c = ioctl(dev, SNDCTL_DSP_SETTRIGGER, &x);
 if(c<0) return c;
#endif //POKA
 return 0;
}

/* Start/stop DAC */

int xdspEnableOutput(int dev, int Enable)
{
 int x, c;
#if POKA

 c = ioctl(dev, SNDCTL_DSP_GETTRIGGER, &x);
 if(c<0) return c;
 if(Enable) x |= PCM_ENABLE_OUTPUT;
 else x &= PCM_ENABLE_OUTPUT;
 c = l2_device_ioctl(dev, SNDCTL_DSP_SETTRIGGER, &x);
 if(c<0) return c;
#endif //POKA
 return 0;
}

/* Read DSP Program Memory */

int xdspReadPM(int dev, int address, int words, PM_WORD *data)
{
 copr_buffer cbuf;
 int l, c, x;
#if POKA

 l = words;
 while(l>0)
 {
  c = 1000;
  if(l<c) c = l;
  cbuf.command = 0;
  cbuf.offs = address;
  cbuf.len = c * sizeof(PM_WORD);
  x = l2_device_ioctl(dev, SNDCTL_COPR_LOAD, &cbuf);
  if(x<0) return x;
  memcpy(cbuf.data, data, cbuf.len);
  cbuf.offs += c;
  data += c;
  l -= c;
 }
#endif //POKA
 return 0;
}

/* Write DSP Program Memory */

int xdspWritePM(int dev, int address, int words, PM_WORD *data)
{
 copr_buffer cbuf;
 int l, c, x;
#if POKA

 l = words;
 while(l>0)
 {
  c = 1000;
  if(l<c) c = l;
  cbuf.command = 1;
  cbuf.offs = address;
  cbuf.len = c * sizeof(PM_WORD);
  memcpy(cbuf.data, data, cbuf.len);
  x = l2_device_ioctl(dev, SNDCTL_COPR_LOAD, &cbuf);
  if(x<0) return x;
  cbuf.offs += c;
  data += c;
  l -= c;
 }
#endif //POKA
 return 0;
}

/* Read DSP Data Memory */

int xdspReadDM(int dev, int address, int words, DM_WORD *data)
{
 copr_buffer cbuf;
 int l, c, x;
#if POKA

 l = words;
 while(l>0)
 {
  c = 1000;
  if(l<c) c = l;
  cbuf.command = 2;
  cbuf.offs = address;
  cbuf.len = c * sizeof(DM_WORD);
  x = l2_device_ioctl(dev, SNDCTL_COPR_LOAD, &cbuf);
  if(x<0) return x;
  memcpy(cbuf.data, data, cbuf.len);
  cbuf.offs += c;
  data += c;
  l -= c;
 }
#endif //POKA
 return 0;
}

/* Write DSP Data Memory */

int xdspWriteDM(int dev, int address, int words, DM_WORD *data)
{
 copr_buffer cbuf;
 int l, c, x;
#if POKA

 l = words;
 while(l>0)
 {
  c = 1000;
  if(l<c) c = l;
  cbuf.command = 3;
  cbuf.offs = address;
  cbuf.len = c * sizeof(DM_WORD);
  memcpy(cbuf.data, data, cbuf.len);
  x = l2_device_ioctl(dev, SNDCTL_COPR_LOAD, &cbuf);
  if(x<0) return x;
  cbuf.offs += c;
  data += c;
  l -= c;
 }
#endif //POKA
 return 0;
}

/* Get board description */

int xdspGetInfo(int dev, void *bd)
{
   if(l2_device_ioctl(dev, XDSP_GET_INFO,(unsigned) bd))
                                       return -1;
 return 0;
}

int xdspGetVar(int dev, int address)
{
 copr_debug_buf cdbuf;

 cdbuf.command = 0;
 cdbuf.parm1 = address;
 cdbuf.parm2 = -1;
 cdbuf.flags = 0;
 cdbuf.len = 0;
#if POKA
 if(l2_device_ioctl(dev, SNDCTL_COPR_RDATA, &cdbuf)) return -1;
 return cdbuf.parm2;
#endif //POKA
}

int xdspSetVar(int dev, int address, int val)
{
 copr_debug_buf cdbuf;
#if POKA

 cdbuf.command = 0;
 cdbuf.parm1 = address;
 cdbuf.parm2 = val;
 cdbuf.flags = 0;
 cdbuf.len = 0;
 return ioctl(dev, SNDCTL_COPR_RDATA, &cdbuf);
#endif //POKA
}

int xdspBiosTest(int dev)
{
#if POKA
 if(l2_device_ioctl(dev, XDSP_TEST)) return -1;
 return 0;
#endif //POKA
}

/* Set input buffer size */

int xdspSetInBuf(int dev, long Size)
{
#if POKA
 return l2_device_ioctl(dev, XDSP_SETIBUF, &Size);
#endif //POKA
}

/* Set output buffer size */

int xdspSetOutBuf(int dev, long Size)
{
#if POKA
 return l2_device_ioctl(dev, XDSP_SETOBUF, &Size);
#endif //POKA
}

/* Enable first N channels */

int xdspSetChannels(int dev, int N)
{
 int c;
#if POKA

 c = l2_device_ioctl(dev, SNDCTL_DSP_CHANNELS, &N);
 if(c<0) return c;
#endif //POKA
 return N;
}

/* Set channels from table */

int xdspSetChannelTab(int dev, int N, int *Tab)
{
 channel_info ci;
 int c, x;
#if POKA

 for(c=0; c<N; c++)
 {
  ci.number = c;
  ci.log_channel = Tab[c];
  x = l2_device_ioctl(dev, XDSP_SET_CHANNEL, &ci);
  if(x) return x;
 }
 return l2_device_ioctl(dev, SNDCTL_DSP_CHANNELS, &N);
#endif //POKA
}

/* Set frame rate */

int xdspSetSpeed(int dev, int *Rate)
{
#if POKA
 return l2_device_ioctl(dev, SNDCTL_DSP_SPEED, Rate);
#endif //POKA
}

/* Set frame timing */

int xdspSetTiming(int dev, frame_timing *ft)
{
#if POKA
 return l2_device_ioctl(dev, XDSP_SET_TIMING, ft);
#endif //POKA
}

/* Set sync mode */

int xdspSetSync(int dev, sync_mode *sm)
{
#if POKA
 return ioctl(dev, XDSP_SYNC_MODE, sm);
#endif //POKA
}

/* Get input buffer position */

int xdspGetInPtr(int dev)
{
#if POKA
 struct count_info ci;

 if(l2_device_ioctl(dev, SNDCTL_DSP_GETIPTR, &ci)) return -1;
 return ci.ptr;
#endif //POKA
}

/* Get output buffer position */

int xdspGetOutPtr(int dev)
{
 struct count_info ci;

#if POKA
 if(ioctl(dev, SNDCTL_DSP_GETOPTR, &ci)) return -1;
 return ci.ptr;
#endif //POKA
}

/* Map buffer */

void *xdspMapBuf(int dev, int stream, long Offset, long Size)
{
 void *p;
 int prot;
#if POKA

 prot = PROT_READ;
 if(stream==0) prot |= PROT_WRITE;
 p = mmap(0, Size, prot, MAP_SHARED, dev, Offset);
 if(p==MAP_FAILED) return 0;
 return p;
#endif //POKA
}

/* Unmap buffer */

int xdspUnMapBuf(int dev, void *start, long length)
{
#if POKA
 return munmap(start, length);
#endif //POKA
}

int xdspStart(int dev, int x)
{
 copr_debug_buf cdbuf;
#if POKA

 cdbuf.command = 0;
 cdbuf.parm1 = x;
 cdbuf.parm2 = 0;
 cdbuf.flags = 0;
 cdbuf.len = 0;
 return l2_device_ioctl(dev, SNDCTL_COPR_RUN, &cdbuf);
#endif //POKA
}

int xdspLoad(int dev, int cmd, void *buf, int nwords, int wsize, int addr)
{
  copr_buffer cbuf;
  int wpb, x, c;
  char *p;

  wpb = sizeof(cbuf.data) / wsize;  /* Number of words in block */
  p = (char *) buf;
  cbuf.flags = CPF_FIRST;
  while(nwords > 0)
  {
       c = wpb;
     if(nwords < c) c = nwords;
     if(c == nwords) cbuf.flags |= CPF_LAST;
     cbuf.command = cmd;
     cbuf.offs = addr;
     cbuf.len = c * wsize;
     memcpy(cbuf.data, p, cbuf.len);
     x = l2_device_ioctl(dev, SNDCTL_COPR_LOAD, (unsigned int)&cbuf);
     if(x)
       return x;
     memcpy(p, cbuf.data, cbuf.len);
     cbuf.flags &= ~CPF_FIRST;
     addr += c;
     p += cbuf.len;
     nwords -= c;
  }
  return 0;
}

int xdspGetData(int dev)
{
 copr_debug_buf cdbuf;
 int c;
#if POKA

 cdbuf.command = 0;
 cdbuf.parm1 = 0;
 cdbuf.parm2 = 0;
 cdbuf.flags = 0;
 cdbuf.len = 0;
 c = l2_device_ioctl(dev, SNDCTL_COPR_RDATA, &cdbuf);
 if(c<0) return c;
 return cdbuf.parm1;
#endif //POKA
}

int xdspPutData(int dev, int x)
{
 copr_debug_buf cdbuf;
#if POKA

 cdbuf.command = 0;
 cdbuf.parm1 = x;
 cdbuf.parm2 = 0;
 cdbuf.flags = 0;
 cdbuf.len = 0;
 return l2_device_ioctl(dev, SNDCTL_COPR_WDATA, &cdbuf);
#endif //POKA
}

int xdspReset(int dev)
{
 return l2_device_ioctl(dev, SNDCTL_COPR_RESET);
}

int xdspGetId(int dev)
{
 int x, c;

 c = -1;
 x = l2_device_ioctl(dev, XDSP_ID, (unsigned int) &c);
 if(x) return x;
 return c;
}

int xdspTest(int dev, int c)
{
 int x;

 x = l2_device_ioctl(dev, XDSP_TEST,(unsigned int) &c);
 if(x) return x;
 return c;
}
